/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.jmx.api.mock.monitoring.component.se.flowable;

import java.util.Map;

import org.ow2.petals.jmx.api.api.monitoring.component.framework.exception.ComponentMonitoringServiceErrorException;
import org.ow2.petals.jmx.api.api.monitoring.component.se.flowable.DatabaseConnectionPoolMetrics;
import org.ow2.petals.jmx.api.api.monitoring.component.se.flowable.SeFlowableMonitoringServiceClient;
import org.ow2.petals.jmx.api.api.monitoring.object.ThreadPoolMetrics;
import org.ow2.petals.jmx.api.mock.junit.monitoring.component.se.flowable.DatabaseConnectionPoolMetricsMock;
import org.ow2.petals.jmx.api.mock.junit.monitoring.object.ThreadPoolMetricsMock;
import org.ow2.petals.jmx.api.mock.monitoring.component.framework.ComponentMonitoringServiceClientMock;

/**
 * Mock of {@link SeFlowableMonitoringServiceClient} for unit tests of application using the Petals JMX API.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class SeFlowableMonitoringServiceClientMock extends ComponentMonitoringServiceClientMock
        implements SeFlowableMonitoringServiceClient {

    //
    // Process definitions
    //

    /**
     * Value returns by the method {@link #getProcessDefinitions()}. Must be set by the unit test if needed.
     */
    public static Map<String, Long[]> PROCESS_DEFINITIONS_INFORMATION = null;

    //
    // Asynchronous job executor thread pool
    //

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getActiveThreadsMaxSize()}. Must be set by the unit test
     * if needed.
     */
    public static Long ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getActiveThreadsSize()}. Must be set by the unit test if
     * needed.
     */
    public static Long ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getIdleThreadsMaxSize()}. Must be set by the unit test
     * if needed.
     */
    public static Long ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getIdleThreadsSize()}. Must be set by the unit test if
     * needed.
     */
    public static Long ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getMaxSize()}. Must be set by the unit test if needed.
     */
    public static Long ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getMinSize()}. Must be set by the unit test if needed.
     */
    public static Long ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getEnqueuedRequestsSize()}. Must be set by the unit test
     * if needed.
     */
    public static Long ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT = null;

    /**
     * Value returns by the method {@link ThreadPoolMetricsMock#getEnqueuedRequestsMaxSize()}. Must be set by the unit
     * test if needed.
     */
    public static Long ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX = null;

    //
    // Database connection pool
    //

    /**
     * Value returns by the method {@link DatabaseConnectionPoolMetricsMock#getActiveConnectionsMaxSize()}. Must be set
     * by the unit test if needed.
     */
    public static Long DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX = null;

    /**
     * Value returns by the method {@link DatabaseConnectionPoolMetricsMock#getActiveConnectionsSize()}. Must be set by
     * the unit test if needed.
     */
    public static Long DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT = null;

    /**
     * Value returns by the method {@link DatabaseConnectionPoolMetricsMock#getIdleConnectionsMaxSize()}. Must be set by
     * the unit test if needed.
     */
    public static Long DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX = null;

    /**
     * Value returns by the method {@link DatabaseConnectionPoolMetricsMock#getIdleConnectionsSize()}. Must be set by
     * the unit test if needed.
     */
    public static Long DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT = null;

    /**
     * Value returns by the method {@link DatabaseConnectionPoolMetricsMock#getMaxActiveSize()}. Must be set by the unit
     * test if needed.
     */
    public static Long DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE = null;

    /**
     * Value returns by the method {@link DatabaseConnectionPoolMetricsMock#getMaxIdleSize()}. Must be set by the unit
     * test if needed.
     */
    public static Long DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE = null;

    public SeFlowableMonitoringServiceClientMock(final String componentName) {
        super(componentName);
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Mock dedicated processing: If the component name is {@value #IDENTIFIER_FOR_COMPONENT_WITH_ERROR} the exception
     * {@link ComponentMonitoringServiceErrorException} is thrown with the message
     * {@value #ERROR_MESSAGE_GETTING_MONITORING_VALUES}. Otherwise the content of following values are returned:
     * <ul>
     * <li>{@link #ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE},</li>
     * <li>{@link #ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE},</li>
     * <li>{@link #ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT},</li>
     * <li>{@link #ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX},</li>
     * <li>{@link #ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT},</li>
     * <li>{@link #ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX},</li>
     * <li>{@link #ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT},</li>
     * <li>{@link #ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX}.</li>
     * </ul>
     * </p>
     */
    @Override
    public ThreadPoolMetrics getAsyncExecutorThreadPoolMetrics() throws ComponentMonitoringServiceErrorException {

        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(this.componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {

            return new ThreadPoolMetricsMock(ASYNC_EXECUTOR_THREAD_POOL_MIN_SIZE, ASYNC_EXECUTOR_THREAD_POOL_MAX_SIZE,
                    ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_CURRENT, ASYNC_EXECUTOR_THREAD_POOL_ACTIVE_THREAD_MAX,
                    ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_CURRENT, ASYNC_EXECUTOR_THREAD_POOL_IDLE_THREAD_MAX,
                    ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_CURRENT,
                    ASYNC_EXECUTOR_THREAD_POOL_ENQUEUED_REQUEST_MAX);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Mock dedicated processing: If the component name is {@value #IDENTIFIER_FOR_COMPONENT_WITH_ERROR} the exception
     * {@link ComponentMonitoringServiceErrorException} is thrown with the message
     * {@value #ERROR_MESSAGE_GETTING_MONITORING_VALUES}. Otherwise the content of
     * {@link #PROCESS_DEFINITIONS_INFORMATION} is returned.
     * </p>
     */
    @Override
    public Map<String, Long[]> getProcessDefinitions() throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(this.componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {
            return PROCESS_DEFINITIONS_INFORMATION;
        }
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Mock dedicated processing: If the component name is {@value #IDENTIFIER_FOR_COMPONENT_WITH_ERROR} the exception
     * {@link ComponentMonitoringServiceErrorException} is thrown with the message
     * {@value #ERROR_MESSAGE_GETTING_MONITORING_VALUES}. Otherwise the content of following values are returned:
     * <ul>
     * <li>{@link #DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE},</li>
     * <li>{@link #DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE},</li>
     * <li>{@link #DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT},</li>
     * <li>{@link #DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX},</li>
     * <li>{@link #DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT},</li>
     * <li>{@link #DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX}.</li>
     * </ul>
     * </p>
     */
    @Override
    public DatabaseConnectionPoolMetrics getDatabaseConnectionPoolMetrics()
            throws ComponentMonitoringServiceErrorException {
        if (IDENTIFIER_FOR_COMPONENT_WITH_ERROR.equals(this.componentName)) {
            throw new ComponentMonitoringServiceErrorException(ERROR_MESSAGE_GETTING_MONITORING_VALUES);
        } else {

            return new DatabaseConnectionPoolMetricsMock(DATABASE_CONNECTION_POOL_MAX_ACTIVE_SIZE,
                    DATABASE_CONNECTION_POOL_MAX_IDLE_SIZE, DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_CURRENT,
                    DATABASE_CONNECTION_POOL_ACTIVE_CONNECTIONS_MAX, DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_CURRENT,
                    DATABASE_CONNECTION_POOL_IDLE_CONNECTIONS_MAX);
        }
    }

}
